import pysicgl


def test_builtin_compositors():
    expected_compositors = (
        "DIRECT_SET",
        "DIRECT_CLEAR",
        "DIRECT_NONE",
        "BIT_AND",
        "BIT_OR",
        "BIT_XOR",
        "BIT_NAND",
        "BIT_NOR",
        "BIT_XNOR",
        # # These bitwise compositors are not implemented yet in sicgl.
        # "BIT_NOT_SOURCE",
        # "BIT_NOT_DESTINATION",
        "CHANNEL_MIN",
        "CHANNEL_MAX",
        "CHANNEL_SUM",
        "CHANNEL_DIFF",
        "CHANNEL_DIFF_REVERSE",
        "CHANNEL_MULTIPLY",
        "CHANNEL_DIVIDE",
        "CHANNEL_DIVIDE_REVERSE",
        "CHANNEL_SUM_CLAMPED",
        "CHANNEL_DIFF_CLAMPED",
        "CHANNEL_DIFF_REVERSE_CLAMPED",
        "CHANNEL_MULTIPLY_CLAMPED",
        "CHANNEL_DIVIDE_CLAMPED",
        "CHANNEL_DIVIDE_REVERSE_CLAMPED",
        "ALPHA_CLEAR",
        "ALPHA_COPY",
        "ALPHA_DESTINATION",
        "ALPHA_SOURCE_OVER",
        "ALPHA_DESTINATION_OVER",
        "ALPHA_SOURCE_IN",
        "ALPHA_DESTINATION_IN",
        "ALPHA_SOURCE_OUT",
        "ALPHA_DESTINATION_OUT",
        "ALPHA_SOURCE_ATOP",
        "ALPHA_DESTINATION_ATOP",
        "ALPHA_XOR",
        "ALPHA_LIGHTER",
    )

    for compositor_name in expected_compositors:
        assert hasattr(pysicgl.composition, compositor_name)
        compositor = getattr(pysicgl.composition, compositor_name)
        assert isinstance(compositor, pysicgl.Compositor)



import pytest
import pysicgl


def test_module_exists():
    assert hasattr(pysicgl, "functional")


def test_has_gamma_correct():
    assert hasattr(pysicgl.functional, "gamma_correct")


def test_has_get_pixel_at_offset():
    assert hasattr(pysicgl.functional, "get_pixel_at_offset")


def test_has_get_pixel_at_coordinates():
    assert hasattr(pysicgl.functional, "get_pixel_at_coordinates")



import pytest
import pysicgl


DEFAULT_COLORS = [2]
DEFAULT_INTERPOLATOR = pysicgl.interpolation.CONTINUOUS_CIRCULAR


def test_initialization():
    sequence = pysicgl.ColorSequence(
        colors=DEFAULT_COLORS, interpolator=DEFAULT_INTERPOLATOR
    )


@pytest.mark.skip(reason="Not implemented")
def test_has_len():
    sequence = pysicgl.ColorSequence(
        colors=DEFAULT_COLORS, interpolator=DEFAULT_INTERPOLATOR
    )
    assert hasattr(sequence, "__len__")


def test_len():
    sequence = pysicgl.ColorSequence(
        colors=DEFAULT_COLORS, interpolator=DEFAULT_INTERPOLATOR
    )
    assert len(sequence) == len(DEFAULT_COLORS)


def test_subscr():
    sequence = pysicgl.ColorSequence(
        colors=DEFAULT_COLORS, interpolator=DEFAULT_INTERPOLATOR
    )
    for idx in range(len(DEFAULT_COLORS)):
        assert sequence[idx] == DEFAULT_COLORS[idx]


@pytest.mark.skip(reason="Not implemented")
def test_length():
    sequence = pysicgl.ColorSequence(
        colors=DEFAULT_COLORS, interpolator=DEFAULT_INTERPOLATOR
    )
    assert len(sequence) == 0

    sequence.colors = ((0, 0, 0, 0),)
    assert len(sequence) == 1

    sequence.colors = ((0, 0, 0, 0), (0, 0, 0, 0))
    assert len(sequence) == 2


def test_iterator():
    sequence = pysicgl.ColorSequence(
        colors=DEFAULT_COLORS, interpolator=DEFAULT_INTERPOLATOR
    )
    assert hasattr(sequence, "__iter__")
    assert iter(sequence) is sequence
    assert len(list(sequence)) == len(DEFAULT_COLORS)

    for color in sequence:
        assert color == DEFAULT_COLORS[0]



import pytest
import pysicgl


@pytest.fixture
def interface():
    WIDTH = 1
    HEIGHT = 1
    display_screen = pysicgl.Screen((WIDTH, HEIGHT))
    display_memory = pysicgl.allocate_pixel_memory(display_screen.pixels)
    _interface = pysicgl.Interface(display_screen, display_memory)
    return _interface



# vector utilities
# vectors are fixed-length iterables with scalar numeric elements

import math


def vec_add(*vectors):
    return tuple(sum(elements) for elements in zip(*vectors))


def vec_magnitude(vector):
    return math.sqrt(sum(math.pow(element, 2) for element in vector))


def vec_normalize(vector):
    magnitude = vec_magnitude(vector)
    return tuple(element / magnitude for element in vector)



import pysicgl

# create a screen definition with extent (WIDTH, HEIGHT)
WIDTH = 256
HEIGHT = 128
display_screen = pysicgl.Screen((WIDTH, HEIGHT))

# allocate memory for the interface using the number of
# pixels in the display screen
display_memory = pysicgl.allocate_pixel_memory(display_screen.pixels)

# create an interface which controls access to this
# memory. the screen definition is used to inform
# geometrical constraints.
display = pysicgl.Interface(display_screen, display_memory)

# create a orange-red color using a 4-tuple of RGBA components
color = pysicgl.color.from_rgba((255, 128, 3, 0))

# draw a pixel directly to the interface origin
# the coordinates are given in the interface-relative system
display.interface_pixel(color, (0, 0))

# show the fist pixel as memory
print(list(display.memory[0:4]))



from setuptools import setup, Extension, find_packages
from pathlib import Path, PurePath

# source files for sicgl
sicgl_root_dir = "third-party/sicgl"
sicgl_include_dirs = list(
    str(PurePath(sicgl_root_dir, include))
    for include in [
        "include",
    ]
)
sicgl_sources = list(
    str(PurePath(sicgl_root_dir, "src", source))
    for source in [
        "compositors/alpha.c",
        "compositors/bitwise.c",
        "compositors/channelwise.c",
        "compositors/direct.c",
        "domain/global.c",
        "domain/interface.c",
        "domain/screen.c",
        "private/direct.c",
        "private/interpolation.c",
        "blend.c",
        "blenders.c",
        "blit.c",
        "color_sequence.c",
        "compose.c",
        "field.c",
        "gamma.c",
        "interface.c",
        "iter.c",
        "screen.c",
        "translate.c",
        "unity_color.c",
    ]
)

pysicgl_root_dir = "."
pysicgl_include_dirs = list(
    str(PurePath(pysicgl_root_dir, include))
    for include in [
        "include",
    ]
)
pysicgl_sources = list(
    str(PurePath(pysicgl_root_dir, "src", source))
    for source in [
        "submodules/composition/module.c",
        "submodules/functional/drawing/global.c",
        "submodules/functional/drawing/interface.c",
        "submodules/functional/drawing/screen.c",
        "submodules/functional/color.c",
        "submodules/functional/color_correction.c",
        "submodules/functional/module.c",
        "submodules/functional/operations.c",
        "submodules/interpolation/module.c",
        "types/color_sequence/type.c",
        "types/color_sequence_interpolator/type.c",
        "types/compositor/type.c",
        "types/scalar_field/type.c",
        "types/interface/type.c",
        "types/screen/type.c",
        "module.c",
    ]
)

sicgl_core = Extension(
    "pysicgl._core",
    include_dirs=[*pysicgl_include_dirs, *sicgl_include_dirs],
    sources=[*pysicgl_sources, *sicgl_sources],
    extra_compile_args=[
      # "-Werror",
      # "-Wall", "-Wextra", "-pedantic",
      # "-Wno-missing-field-initializers", "-Wno-sign-compare", "-Wno-sometimes-uninitialized",
    ],
)

setup(
    ext_modules=[sicgl_core],
    packages=find_packages(where="packages"),
    package_dir={'': 'packages'},
    setup_requires=["setuptools_scm"],
)
